@@ -40,6 +40,9 @@
 
 #include "htp_private.h"
 
+static void htp_tx_req_destroy_decompressors(htp_connp_t *connp);
+static htp_status_t htp_tx_req_process_body_data_decompressor_callback(htp_tx_data_t *d);
+
 static bstr *copy_or_wrap_mem(const void *data, size_t len, enum htp_alloc_strategy_t alloc) {
     if (data == NULL) return NULL;
 
@@ -362,6 +365,35 @@ static htp_status_t htp_tx_process_request_headers(htp_tx_t *tx) {
 
     htp_status_t rc = HTP_OK;
 
+    if (tx->connp->cfg->request_decompression_enabled) {
+        tx->request_content_encoding = HTP_COMPRESSION_NONE;
+        htp_header_t *ce = htp_table_get_c(tx->request_headers, "content-encoding");
+        if (ce != NULL) {
+            /* fast paths: regular gzip and friends */
+            if ((bstr_cmp_c_nocasenorzero(ce->value, "gzip") == 0) ||
+                (bstr_cmp_c_nocasenorzero(ce->value, "x-gzip") == 0)) {
+                tx->request_content_encoding = HTP_COMPRESSION_GZIP;
+            } else if ((bstr_cmp_c_nocasenorzero(ce->value, "deflate") == 0) ||
+                       (bstr_cmp_c_nocasenorzero(ce->value, "x-deflate") == 0)) {
+                tx->request_content_encoding = HTP_COMPRESSION_DEFLATE;
+            } else if (bstr_cmp_c_nocasenorzero(ce->value, "lzma") == 0) {
+                tx->request_content_encoding = HTP_COMPRESSION_LZMA;
+            }
+            //ignore other cases such as inflate, ot multiple layers
+            if ((tx->request_content_encoding != HTP_COMPRESSION_NONE))
+            {
+                if (tx->connp->req_decompressor != NULL) {
+                    htp_tx_req_destroy_decompressors(tx->connp);
+                }
+                tx->connp->req_decompressor = htp_gzip_decompressor_create(tx->connp, tx->request_content_encoding);
+                if (tx->connp->req_decompressor == NULL)
+                    return HTP_ERROR;
+
+                tx->connp->req_decompressor->callback = htp_tx_req_process_body_data_decompressor_callback;
+            }
+        }
+    }
+
     htp_header_t *cl = htp_table_get_c(tx->request_headers, "content-length");
     htp_header_t *te = htp_table_get_c(tx->request_headers, "transfer-encoding");
 
@@ -373,7 +405,7 @@ static htp_status_t htp_tx_process_request_headers(htp_tx_t *tx) {
         //      (2.2.22 on Ubuntu 12.04 LTS) instead errors out with "Unknown Transfer-Encoding: identity".
         //      And it behaves strangely, too, sending a 501 and proceeding to process the request
         //      (e.g., PHP is run), but without the body. It then closes the connection.
-        if (bstr_cmp_c_nocase(te->value, "chunked") != 0) {
+        if (htp_header_has_token(bstr_ptr(te->value), bstr_len(te->value), (unsigned char*) "chunked") != HTP_OK) {
             // Invalid T-E header value.
             tx->request_transfer_coding = HTP_CODING_INVALID;
             tx->flags |= HTP_REQUEST_INVALID_T_E;
@@ -421,7 +453,7 @@ static htp_status_t htp_tx_process_request_headers(htp_tx_t *tx) {
         }
 
         // Get the body length.
-        tx->request_content_length = htp_parse_content_length(cl->value);
+        tx->request_content_length = htp_parse_content_length(cl->value, tx->connp);
         if (tx->request_content_length < 0) {
             tx->request_transfer_coding = HTP_CODING_INVALID;
             tx->flags |= HTP_REQUEST_INVALID_C_L;
@@ -553,10 +585,7 @@ static htp_status_t htp_tx_process_request_headers(htp_tx_t *tx) {
     rc = htp_hook_run_all(tx->connp->cfg->hook_request_headers, tx);
     if (rc != HTP_OK) return rc;
 
-    // We cannot proceed if the request is invalid.
-    if (tx->flags & HTP_REQUEST_INVALID) {
-        return HTP_ERROR;
-    }
+    // We still proceed if the request is invalid.
 
     return HTP_OK;
 }
@@ -574,20 +603,49 @@ htp_status_t htp_tx_req_process_body_data_ex(htp_tx_t *tx, const void *data, siz
     // NULL data is allowed in this private function; it's
     // used to indicate the end of request body.
 
-    // Keep track of the body length.
-    tx->request_entity_len += len;
-
     // Send data to the callbacks.
 
     htp_tx_data_t d;
     d.tx = tx;
     d.data = (unsigned char *) data;
     d.len = len;
+    d.is_last = (data == NULL && len == 0);
 
-    htp_status_t rc = htp_req_run_hook_body_data(tx->connp, &d);
-    if (rc != HTP_OK) {
-        htp_log(tx->connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0, "Request body data callback returned error (%d)", rc);
-        return HTP_ERROR;
+    switch(tx->request_content_encoding) {
+        case HTP_COMPRESSION_UNKNOWN:
+        case HTP_COMPRESSION_NONE:
+            // When there's no decompression, request_entity_len.
+            // is identical to request_message_len.
+            tx->request_entity_len += d.len;
+            htp_status_t rc = htp_req_run_hook_body_data(tx->connp, &d);
+            if (rc != HTP_OK) {
+                htp_log(tx->connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0, "Request body data callback returned error (%d)", rc);
+                return HTP_ERROR;
+            }
+            break;
+
+        case HTP_COMPRESSION_GZIP:
+        case HTP_COMPRESSION_DEFLATE:
+        case HTP_COMPRESSION_LZMA:
+            // In severe memory stress these could be NULL
+            if (tx->connp->req_decompressor == NULL)
+                return HTP_ERROR;
+
+            // Send data buffer to the decompressor.
+            htp_gzip_decompressor_decompress(tx->connp->req_decompressor, &d);
+
+            if (data == NULL) {
+                // Shut down the decompressor, if we used one.
+                htp_tx_req_destroy_decompressors(tx->connp);
+            }
+            break;
+
+        default:
+            // Internal error.
+            htp_log(tx->connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0,
+                    "[Internal Error] Invalid tx->request_content_encoding value: %d",
+                    tx->request_content_encoding);
+            return HTP_ERROR;
     }
 
     return HTP_OK;
@@ -684,11 +742,18 @@ htp_status_t htp_tx_state_response_line(htp_tx_t *tx) {
     #endif
 
     // Is the response line valid?
-    if ((tx->response_protocol_number == HTP_PROTOCOL_INVALID)
-            || (tx->response_status_number == HTP_STATUS_INVALID)
+    if (tx->response_protocol_number == HTP_PROTOCOL_INVALID) {
+        htp_log(tx->connp, HTP_LOG_MARK, HTP_LOG_WARNING, 0,
+                "Invalid response line: invalid protocol");
+        tx->flags |= HTP_STATUS_LINE_INVALID;
+    }
+    if ((tx->response_status_number == HTP_STATUS_INVALID)
             || (tx->response_status_number < HTP_VALID_STATUS_MIN)
             || (tx->response_status_number > HTP_VALID_STATUS_MAX)) {
-        htp_log(tx->connp, HTP_LOG_MARK, HTP_LOG_WARNING, 0, "Invalid response line.");
+        htp_log(tx->connp, HTP_LOG_MARK, HTP_LOG_WARNING, 0,
+                "Invalid response line: invalid response status %d.",
+                tx->response_status_number);
+        tx->response_status_number = HTP_STATUS_INVALID;
         tx->flags |= HTP_STATUS_LINE_INVALID;
     }
 
@@ -749,24 +814,90 @@ htp_status_t htp_tx_res_set_headers_clear(htp_tx_t *tx) {
     return HTP_OK;
 }
 
-void htp_connp_destroy_decompressors(htp_connp_t *connp) {
-    htp_decompressor_t *comp = connp->out_decompressor;
-    while (comp) {
-        htp_decompressor_t *next = comp->next;
-        comp->destroy(comp);
-        comp = next;
-    }
-    connp->out_decompressor = NULL;
-}
-
 /** \internal
  *
  * Clean up decompressor(s).
  *
  * @param[in] tx
  */
-static void htp_tx_res_destroy_decompressors(htp_tx_t *tx) {
-    htp_connp_destroy_decompressors(tx->connp);
+static void htp_tx_res_destroy_decompressors(htp_connp_t *connp) {
+    htp_decompressor_t *comp = connp->out_decompressor;
+    while (comp) {
+        htp_decompressor_t *next = comp->next;
+        htp_gzip_decompressor_destroy(comp);
+        comp = next;
+    }
+    connp->out_decompressor = NULL;
+}
+
+static void htp_tx_req_destroy_decompressors(htp_connp_t *connp) {
+    htp_decompressor_t *comp = connp->req_decompressor;
+    while (comp) {
+        htp_decompressor_t *next = comp->next;
+        htp_gzip_decompressor_destroy(comp);
+        comp = next;
+    }
+    connp->req_decompressor = NULL;
+}
+
+void htp_connp_destroy_decompressors(htp_connp_t *connp) {
+    htp_tx_res_destroy_decompressors(connp);
+    htp_tx_req_destroy_decompressors(connp);
+}
+
+static htp_status_t htp_timer_track(int32_t *time_spent, struct timeval * after, struct timeval *before) {
+    if (after->tv_sec < before->tv_sec) {
+        return HTP_ERROR;
+    } else if (after->tv_sec == before->tv_sec) {
+        if (after->tv_usec < before->tv_usec) {
+            return HTP_ERROR;
+        }
+        *time_spent += after->tv_usec - before->tv_usec;
+    } else {
+        *time_spent += (after->tv_sec - before->tv_sec) * 1000000 + after->tv_usec - before->tv_usec;
+    }
+    return HTP_OK;
+}
+
+static htp_status_t htp_tx_req_process_body_data_decompressor_callback(htp_tx_data_t *d) {
+    if (d == NULL) return HTP_ERROR;
+
+    #if HTP_DEBUG
+    fprint_raw_data(stderr, __func__, d->data, d->len);
+    #endif
+
+    // Keep track of actual request body length.
+    d->tx->request_entity_len += d->len;
+
+    // Invoke all callbacks.
+    htp_status_t rc = htp_req_run_hook_body_data(d->tx->connp, d);
+    if (rc != HTP_OK) return HTP_ERROR;
+    d->tx->connp->req_decompressor->nb_callbacks++;
+    if (d->tx->connp->req_decompressor->nb_callbacks % HTP_COMPRESSION_TIME_FREQ_TEST == 0) {
+        struct timeval after;
+        gettimeofday(&after, NULL);
+        // sanity check for race condition if system time changed
+        if ( htp_timer_track(&d->tx->connp->req_decompressor->time_spent, &after, &d->tx->connp->req_decompressor->time_before) == HTP_OK) {
+            // updates last tracked time
+            d->tx->connp->req_decompressor->time_before = after;
+            if (d->tx->connp->req_decompressor->time_spent > d->tx->connp->cfg->compression_time_limit ) {
+                htp_log(d->tx->connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0,
+                        "Compression bomb: spent %"PRId32" us decompressing",
+                        d->tx->connp->req_decompressor->time_spent);
+                d->tx->connp->req_decompressor->passthrough = 1;
+            }
+        }
+
+    }
+    if (d->tx->request_entity_len > d->tx->connp->cfg->compression_bomb_limit &&
+        d->tx->request_entity_len > HTP_COMPRESSION_BOMB_RATIO * d->tx->request_message_len) {
+        htp_log(d->tx->connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0,
+                "Compression bomb: decompressed %"PRId64" bytes out of %"PRId64,
+                d->tx->request_entity_len, d->tx->request_message_len);
+        return HTP_ERROR;
+    }
+
+    return HTP_OK;
 }
 
 static htp_status_t htp_tx_res_process_body_data_decompressor_callback(htp_tx_data_t *d) {
@@ -782,6 +913,30 @@ static htp_status_t htp_tx_res_process_body_data_decompressor_callback(htp_tx_da
     // Invoke all callbacks.
     htp_status_t rc = htp_res_run_hook_body_data(d->tx->connp, d);
     if (rc != HTP_OK) return HTP_ERROR;
+    d->tx->connp->out_decompressor->nb_callbacks++;
+    if (d->tx->connp->out_decompressor->nb_callbacks % HTP_COMPRESSION_TIME_FREQ_TEST == 0) {
+        struct timeval after;
+        gettimeofday(&after, NULL);
+        // sanity check for race condition if system time changed
+        if ( htp_timer_track(&d->tx->connp->out_decompressor->time_spent, &after, &d->tx->connp->out_decompressor->time_before) == HTP_OK) {
+            // updates last tracked time
+            d->tx->connp->out_decompressor->time_before = after;
+            if (d->tx->connp->out_decompressor->time_spent > d->tx->connp->cfg->compression_time_limit ) {
+                htp_log(d->tx->connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0,
+                        "Compression bomb: spent %"PRId32" us decompressing",
+                        d->tx->connp->out_decompressor->time_spent);
+                d->tx->connp->out_decompressor->passthrough = 1;
+            }
+        }
+
+    }
+    if (d->tx->response_entity_len > d->tx->connp->cfg->compression_bomb_limit &&
+        d->tx->response_entity_len > HTP_COMPRESSION_BOMB_RATIO * d->tx->response_message_len) {
+        htp_log(d->tx->connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0,
+                "Compression bomb: decompressed %"PRId64" bytes out of %"PRId64,
+                d->tx->response_entity_len, d->tx->response_message_len);
+        return HTP_ERROR;
+    }
 
     return HTP_OK;
 }
@@ -815,16 +970,30 @@ htp_status_t htp_tx_res_process_body_data_ex(htp_tx_t *tx, const void *data, siz
     switch (tx->response_content_encoding_processing) {
         case HTP_COMPRESSION_GZIP:
         case HTP_COMPRESSION_DEFLATE:
+        case HTP_COMPRESSION_LZMA:
             // In severe memory stress these could be NULL
-            if (tx->connp->out_decompressor == NULL || tx->connp->out_decompressor->decompress == NULL)
+            if (tx->connp->out_decompressor == NULL)
                 return HTP_ERROR;
 
+            struct timeval after;
+            gettimeofday(&tx->connp->out_decompressor->time_before, NULL);
             // Send data buffer to the decompressor.
-            tx->connp->out_decompressor->decompress(tx->connp->out_decompressor, &d);
+            tx->connp->out_decompressor->nb_callbacks=0;
+            htp_gzip_decompressor_decompress(tx->connp->out_decompressor, &d);
+            gettimeofday(&after, NULL);
+            // sanity check for race condition if system time changed
+            if ( htp_timer_track(&tx->connp->out_decompressor->time_spent, &after, &tx->connp->out_decompressor->time_before) == HTP_OK) {
+                if ( tx->connp->out_decompressor->time_spent > tx->connp->cfg->compression_time_limit ) {
+                    htp_log(tx->connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0,
+                            "Compression bomb: spent %"PRId32" us decompressing",
+                            tx->connp->out_decompressor->time_spent);
+                    tx->connp->out_decompressor->passthrough = 1;
+                }
+            }
 
             if (data == NULL) {
                 // Shut down the decompressor, if we used one.
-                htp_tx_res_destroy_decompressors(tx);
+                htp_tx_res_destroy_decompressors(tx->connp);
             }
             break;
 
@@ -863,6 +1032,8 @@ htp_status_t htp_tx_state_request_complete_partial(htp_tx_t *tx) {
     // Run hook REQUEST_COMPLETE.
     htp_status_t rc = htp_hook_run_all(tx->connp->cfg->hook_request_complete, tx);
     if (rc != HTP_OK) return rc;
+    rc = htp_connp_req_receiver_finalize_clear(tx->connp);
+    if (rc != HTP_OK) return rc;
 
     // Clean-up.
     if (tx->connp->put_file != NULL) {
@@ -1045,6 +1216,10 @@ htp_status_t htp_tx_state_response_complete_ex(htp_tx_t *tx, int hybrid_mode) {
         // Run hook RESPONSE_COMPLETE.
         htp_status_t rc = htp_hook_run_all(tx->connp->cfg->hook_response_complete, tx);
         if (rc != HTP_OK) return rc;
+
+        // Clear the data receivers hook if any
+        rc = htp_connp_res_receiver_finalize_clear(tx->connp);
+        if (rc != HTP_OK) return rc;
     }
 
     if (!hybrid_mode) {
@@ -1099,7 +1274,7 @@ static int get_token(const unsigned char *in, size_t in_len, const char *seps,
     unsigned char **ret_tok_ptr, size_t *ret_tok_len)
 {
     #if HTP_DEBUG
-    fprintf(stderr, "INPUT %u", (uint)in_len);
+    fprintf(stderr, "INPUT %"PRIuMAX, (uintmax_t)in_len);
     fprint_raw_data(stderr, __func__, in, in_len);
     #endif
 
@@ -1127,7 +1302,7 @@ static int get_token(const unsigned char *in, size_t in_len, const char *seps,
     in_len -= i;
 
     #if HTP_DEBUG
-    fprintf(stderr, "INPUT (POST SEP STRIP) %u", (uint)in_len);
+    fprintf(stderr, "INPUT (POST SEP STRIP) %"PRIuMAX, (uintmax_t)in_len);
     fprint_raw_data(stderr, __func__, in, in_len);
     #endif
 
@@ -1159,11 +1334,15 @@ htp_status_t htp_tx_state_response_headers(htp_tx_t *tx) {
     htp_header_t *ce = htp_table_get_c(tx->response_headers, "content-encoding");
     if (ce != NULL) {
         /* fast paths: regular gzip and friends */
-        if ((bstr_cmp_c_nocase(ce->value, "gzip") == 0) || (bstr_cmp_c_nocase(ce->value, "x-gzip") == 0)) {
+        if ((bstr_cmp_c_nocasenorzero(ce->value, "gzip") == 0) ||
+            (bstr_cmp_c_nocasenorzero(ce->value, "x-gzip") == 0)) {
             tx->response_content_encoding = HTP_COMPRESSION_GZIP;
-        } else if ((bstr_cmp_c_nocase(ce->value, "deflate") == 0) || (bstr_cmp_c_nocase(ce->value, "x-deflate") == 0)) {
+        } else if ((bstr_cmp_c_nocasenorzero(ce->value, "deflate") == 0) ||
+                   (bstr_cmp_c_nocasenorzero(ce->value, "x-deflate") == 0)) {
             tx->response_content_encoding = HTP_COMPRESSION_DEFLATE;
-        } else if (bstr_cmp_c_nocase(ce->value, "inflate") == 0) {
+        } else if (bstr_cmp_c_nocasenorzero(ce->value, "lzma") == 0) {
+            tx->response_content_encoding = HTP_COMPRESSION_LZMA;
+        } else if (bstr_cmp_c_nocasenorzero(ce->value, "inflate") == 0) {
             // ignore
         } else {
             /* exceptional cases: enter slow path */
@@ -1200,10 +1379,11 @@ htp_status_t htp_tx_state_response_headers(htp_tx_t *tx) {
     //    supported algorithms.
     if ((tx->response_content_encoding_processing == HTP_COMPRESSION_GZIP) ||
         (tx->response_content_encoding_processing == HTP_COMPRESSION_DEFLATE) ||
+        (tx->response_content_encoding_processing == HTP_COMPRESSION_LZMA) ||
          ce_multi_comp)
     {
         if (tx->connp->out_decompressor != NULL) {
-            htp_tx_res_destroy_decompressors(tx);
+            htp_tx_res_destroy_decompressors(tx->connp);
         }
 
         /* normal case */
@@ -1217,6 +1397,7 @@ htp_status_t htp_tx_state_response_headers(htp_tx_t *tx) {
         } else {
             int layers = 0;
             htp_decompressor_t *comp = NULL;
+            int nblzma = 0;
 
             uint8_t *tok = NULL;
             size_t tok_len = 0;
@@ -1225,7 +1406,7 @@ htp_status_t htp_tx_state_response_headers(htp_tx_t *tx) {
             size_t input_len = bstr_len(ce->value);
 
             #if HTP_DEBUG
-            fprintf(stderr, "INPUT %u", (uint)input_len);
+            fprintf(stderr, "INPUT %"PRIuMAX, (uintmax_t)input_len);
             fprint_raw_data(stderr, __func__, input, input_len);
             #endif
 
@@ -1233,7 +1414,7 @@ htp_status_t htp_tx_state_response_headers(htp_tx_t *tx) {
                    get_token(input, input_len, ", ", &tok, &tok_len))
             {
                 #if HTP_DEBUG
-                fprintf(stderr, "TOKEN %u", (uint)tok_len);
+                fprintf(stderr, "TOKEN %"PRIuMAX, (uintmax_t)tok_len);
                 fprint_raw_data(stderr, __func__, tok, tok_len);
                 #endif
                 enum htp_content_encoding_t cetype = HTP_COMPRESSION_NONE;
@@ -1247,6 +1428,7 @@ htp_status_t htp_tx_state_response_headers(htp_tx_t *tx) {
                     break;
                 }
 
+                nblzma++;
                 if (bstr_util_mem_index_of_c_nocase(tok, tok_len, "gzip") != -1) {
                     if (!(bstr_util_cmp_mem(tok, tok_len, "gzip", 4) == 0 ||
                           bstr_util_cmp_mem(tok, tok_len, "x-gzip", 6) == 0)) {
@@ -1261,7 +1443,14 @@ htp_status_t htp_tx_state_response_headers(htp_tx_t *tx) {
                                 "C-E deflate has abnormal value");
                     }
                     cetype = HTP_COMPRESSION_DEFLATE;
-                } else if (bstr_util_cmp_mem(tok, tok_len, "inflate", 7) == 0) {
+                } else if (bstr_util_cmp_mem(tok, tok_len, "lzma", 4) == 0) {
+                    cetype = HTP_COMPRESSION_LZMA;
+                    if (nblzma > tx->connp->cfg->response_lzma_layer_limit) {
+                        htp_log(tx->connp, HTP_LOG_MARK, HTP_LOG_ERROR, 0,
+                                "Compression bomb: multiple encoding with lzma");
+                        break;
+                    }
+                } else if (bstr_util_cmp_mem(tok, tok_len, "inflate", 7) == 0 || bstr_util_cmp_mem(tok, tok_len, "none", 4) == 0) {
                     cetype = HTP_COMPRESSION_NONE;
                 } else {
                     // continue
@@ -1274,7 +1463,6 @@ htp_status_t htp_tx_state_response_headers(htp_tx_t *tx) {
                         tx->response_content_encoding_processing = cetype;
                         tx->connp->out_decompressor = htp_gzip_decompressor_create(tx->connp, tx->response_content_encoding_processing);
                         if (tx->connp->out_decompressor == NULL) {
-                            free(input);
                             return HTP_ERROR;
                         }
                         tx->connp->out_decompressor->callback = htp_tx_res_process_body_data_decompressor_callback;
@@ -1282,7 +1470,6 @@ htp_status_t htp_tx_state_response_headers(htp_tx_t *tx) {
                     } else {
                         comp->next = htp_gzip_decompressor_create(tx->connp, cetype);
                         if (comp->next == NULL) {
-                            free(input);
                             return HTP_ERROR;
                         }
                         comp->next->callback = htp_tx_res_process_body_data_decompressor_callback;
@@ -1330,10 +1517,6 @@ htp_status_t htp_tx_state_response_start(htp_tx_t *tx) {
      * or a overly long request */
     if (tx->request_method == HTP_M_UNKNOWN && tx->request_uri == NULL && tx->connp->in_state == htp_connp_REQ_LINE) {
         htp_log(tx->connp, HTP_LOG_MARK, HTP_LOG_WARNING, 0, "Request line incomplete");
-
-        if (htp_connp_REQ_LINE_complete(tx->connp) != HTP_OK) {
-            return HTP_ERROR;
-        }
     }
 
     return HTP_OK;
