@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2010 Open Information Security Foundation
+/* Copyright (C) 2007-2023 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -48,7 +48,7 @@ static int mangle_errors = 0;
 
 static char *conf_dirname = NULL;
 
-static int ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel);
+static int ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel, int state);
 
 /* Configuration processing states. */
 enum conf_state {
@@ -92,15 +92,13 @@ ConfYamlSetConfDirname(const char *filename)
     if (ep == NULL) {
         conf_dirname = SCStrdup(".");
         if (conf_dirname == NULL) {
-               FatalError(SC_ERR_FATAL,
-                          "ERROR: Failed to allocate memory while loading configuration.");
+            FatalError("ERROR: Failed to allocate memory while loading configuration.");
         }
     }
     else {
         conf_dirname = SCStrdup(filename);
         if (conf_dirname == NULL) {
-               FatalError(SC_ERR_FATAL,
-                          "ERROR: Failed to allocate memory while loading configuration.");
+            FatalError("ERROR: Failed to allocate memory while loading configuration.");
         }
         conf_dirname[ep - filename] = '\0';
     }
@@ -115,8 +113,7 @@ ConfYamlSetConfDirname(const char *filename)
  *
  * \retval 0 on success, -1 on failure.
  */
-static int
-ConfYamlHandleInclude(ConfNode *parent, const char *filename)
+int ConfYamlHandleInclude(ConfNode *parent, const char *filename)
 {
     yaml_parser_t parser;
     char include_filename[PATH_MAX];
@@ -124,7 +121,7 @@ ConfYamlHandleInclude(ConfNode *parent, const char *filename)
     int ret = -1;
 
     if (yaml_parser_initialize(&parser) != 1) {
-        SCLogError(SC_ERR_CONF_YAML_ERROR, "Failed to initialize YAML parser");
+        SCLogError("Failed to initialize YAML parser");
         return -1;
     }
 
@@ -138,17 +135,15 @@ ConfYamlHandleInclude(ConfNode *parent, const char *filename)
 
     file = fopen(include_filename, "r");
     if (file == NULL) {
-        SCLogError(SC_ERR_FOPEN,
-            "Failed to open configuration include file %s: %s",
-            include_filename, strerror(errno));
+        SCLogError("Failed to open configuration include file %s: %s", include_filename,
+                strerror(errno));
         goto done;
     }
 
     yaml_parser_set_input_file(&parser, file);
 
-    if (ConfYamlParse(&parser, parent, 0, 0) != 0) {
-        SCLogError(SC_ERR_CONF_YAML_ERROR,
-            "Failed to include configuration file %s", filename);
+    if (ConfYamlParse(&parser, parent, 0, 0, 0) != 0) {
+        SCLogError("Failed to include configuration file %s", filename);
         goto done;
     }
 
@@ -171,28 +166,26 @@ done:
  *
  * \retval 0 on success, -1 on failure.
  */
-static int
-ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel)
+static int ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel, int state)
 {
     ConfNode *node = parent;
     yaml_event_t event;
     memset(&event, 0, sizeof(event));
     int done = 0;
-    int state = 0;
     int seq_idx = 0;
     int retval = 0;
     int was_empty = -1;
+    int include_count = 0;
 
     if (rlevel++ > RECURSION_LIMIT) {
-        SCLogError(SC_ERR_CONF_YAML_ERROR, "Recursion limit reached while parsing "
-                "configuration file, aborting.");
+        SCLogError("Recursion limit reached while parsing "
+                   "configuration file, aborting.");
         return -1;
     }
 
     while (!done) {
         if (!yaml_parser_parse(parser, &event)) {
-            SCLogError(SC_ERR_CONF_YAML_ERROR,
-                    "Failed to parse configuration file at line %" PRIuMAX ": %s",
+            SCLogError("Failed to parse configuration file at line %" PRIuMAX ": %s",
                     (uintmax_t)parser->problem_mark.line, parser->problem);
             retval = -1;
             break;
@@ -206,15 +199,15 @@ ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel)
             yaml_version_directive_t *ver =
                 event.data.document_start.version_directive;
             if (ver == NULL) {
-                SCLogError(SC_ERR_CONF_YAML_ERROR, "ERROR: Invalid configuration file.");
-                SCLogError(SC_ERR_CONF_YAML_ERROR,
-                           "The configuration file must begin with the following two lines: %%YAML 1.1 and ---");
+                SCLogError("ERROR: Invalid configuration file.");
+                SCLogError("The configuration file must begin with the following two lines: %%YAML "
+                           "1.1 and ---");
                 goto fail;
             }
             int major = ver->major;
             int minor = ver->minor;
             if (!(major == YAML_VERSION_MAJOR && minor == YAML_VERSION_MINOR)) {
-                SCLogError(SC_ERR_CONF_YAML_ERROR, "ERROR: Invalid YAML version.  Must be 1.1");
+                SCLogError("ERROR: Invalid YAML version.  Must be 1.1");
                 goto fail;
             }
         }
@@ -231,7 +224,25 @@ ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel)
                 goto next;
             }
 
+            /* If the value is unquoted, certain strings in YAML represent NULL. */
+            if ((inseq || state == CONF_VAL) &&
+                    event.data.scalar.style == YAML_PLAIN_SCALAR_STYLE) {
+                if (strlen(value) == 0 || strcmp(value, "~") == 0 || strcmp(value, "null") == 0 ||
+                        strcmp(value, "Null") == 0 || strcmp(value, "NULL") == 0) {
+                    value = NULL;
+                }
+            }
+
             if (inseq) {
+                if (state == CONF_INCLUDE) {
+                    if (value != NULL) {
+                        SCLogInfo("Including configuration file %s.", value);
+                        if (ConfYamlHandleInclude(parent, value) != 0) {
+                            goto fail;
+                        }
+                    }
+                    goto next;
+                }
                 char sequence_node_name[DEFAULT_NAME_LEN];
                 snprintf(sequence_node_name, DEFAULT_NAME_LEN, "%d", seq_idx++);
                 ConfNode *seq_node = NULL;
@@ -264,10 +275,14 @@ ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel)
                         SCFree(seq_node);
                         goto fail;
                     }
-                    seq_node->val = SCStrdup(value);
-                    if (unlikely(seq_node->val == NULL)) {
-                        SCFree(seq_node->name);
-                        goto fail;
+                    if (value != NULL) {
+                        seq_node->val = SCStrdup(value);
+                        if (unlikely(seq_node->val == NULL)) {
+                            SCFree(seq_node->name);
+                            goto fail;
+                        }
+                    } else {
+                        seq_node->val = NULL;
                     }
                 }
                 TAILQ_INSERT_TAIL(&parent->head, seq_node, next);
@@ -284,6 +299,12 @@ ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel)
 
                     if (strcmp(value, "include") == 0) {
                         state = CONF_INCLUDE;
+                        if (++include_count > 1) {
+                            SCLogWarning("Multipline \"include\" fields at the same level are "
+                                         "deprecated and will not work in Suricata 8, please move "
+                                         "to an array of include files: line: %zu",
+                                    parser->mark.line);
+                        }
                         goto next;
                     }
 
@@ -294,46 +315,55 @@ ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel)
                                 Mangle(parent->val);
                         }
                     }
-                    ConfNode *existing = ConfNodeLookupChild(parent, value);
-                    if (existing != NULL) {
-                        if (!existing->final) {
-                            SCLogInfo("Configuration node '%s' redefined.",
-                                existing->name);
-                            ConfNodePrune(existing);
+
+                    if (strchr(value, '.') != NULL) {
+                        node = ConfNodeGetNodeOrCreate(parent, value, 0);
+                        if (node == NULL) {
+                            /* Error message already logged. */
+                            goto fail;
                         }
-                        node = existing;
-                    }
-                    else {
-                        node = ConfNodeNew();
-                        node->name = SCStrdup(value);
-                        if (node->name && strchr(node->name, '_')) {
-                            if (!(parent->name &&
-                                   ((strcmp(parent->name, "address-groups") == 0) ||
-                                    (strcmp(parent->name, "port-groups") == 0)))) {
-                                Mangle(node->name);
-                                if (mangle_errors < MANGLE_ERRORS_MAX) {
-                                    SCLogWarning(SC_WARN_DEPRECATED,
-                                            "%s is deprecated. Please use %s on line %"PRIuMAX".",
-                                            value, node->name, (uintmax_t)parser->mark.line+1);
-                                    mangle_errors++;
-                                    if (mangle_errors >= MANGLE_ERRORS_MAX)
-                                        SCLogWarning(SC_WARN_DEPRECATED, "not showing more "
-                                                "parameter name warnings.");
+                    } else {
+                        ConfNode *existing = ConfNodeLookupChild(parent, value);
+                        if (existing != NULL) {
+                            if (!existing->final) {
+                                SCLogInfo("Configuration node '%s' redefined.", existing->name);
+                                ConfNodePrune(existing);
+                            }
+                            node = existing;
+                        } else {
+                            node = ConfNodeNew();
+                            node->name = SCStrdup(value);
+                            node->parent = parent;
+                            if (node->name && strchr(node->name, '_')) {
+                                if (!(parent->name &&
+                                            ((strcmp(parent->name, "address-groups") == 0) ||
+                                                    (strcmp(parent->name, "port-groups") == 0)))) {
+                                    Mangle(node->name);
+                                    if (mangle_errors < MANGLE_ERRORS_MAX) {
+                                        SCLogWarning(
+                                                "%s is deprecated. Please use %s on line %" PRIuMAX
+                                                ".",
+                                                value, node->name,
+                                                (uintmax_t)parser->mark.line + 1);
+                                        mangle_errors++;
+                                        if (mangle_errors >= MANGLE_ERRORS_MAX)
+                                            SCLogWarning("not showing more "
+                                                         "parameter name warnings.");
+                                    }
                                 }
                             }
+                            TAILQ_INSERT_TAIL(&parent->head, node, next);
                         }
-                        TAILQ_INSERT_TAIL(&parent->head, node, next);
                     }
                     state = CONF_VAL;
                 }
                 else {
-                    if ((tag != NULL) && (strcmp(tag, "!include") == 0)) {
+                    if (value != NULL && (tag != NULL) && (strcmp(tag, "!include") == 0)) {
                         SCLogInfo("Including configuration file %s at "
                             "parent node %s.", value, node->name);
                         if (ConfYamlHandleInclude(node, value) != 0)
                             goto fail;
-                    }
-                    else if (!node->final) {
+                    } else if (!node->final && value != NULL) {
                         if (node->val != NULL)
                             SCFree(node->val);
                         node->val = SCStrdup(value);
@@ -344,7 +374,9 @@ ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel)
         }
         else if (event.type == YAML_SEQUENCE_START_EVENT) {
             SCLogDebug("event.type=YAML_SEQUENCE_START_EVENT; state=%d", state);
-            if (ConfYamlParse(parser, node, 1, rlevel) != 0)
+            /* If we're processing a list of includes, use the current parent. */
+            if (ConfYamlParse(parser, state == CONF_INCLUDE ? parent : node, 1, rlevel,
+                        state == CONF_INCLUDE ? CONF_INCLUDE : 0) != 0)
                 goto fail;
             node->is_seq = 1;
             state = CONF_KEY;
@@ -355,6 +387,10 @@ ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel)
         }
         else if (event.type == YAML_MAPPING_START_EVENT) {
             SCLogDebug("event.type=YAML_MAPPING_START_EVENT; state=%d", state);
+            if (state == CONF_INCLUDE) {
+                SCLogError("Include fields cannot be a mapping: line %zu", parser->mark.line);
+                goto fail;
+            }
             if (inseq) {
                 char sequence_node_name[DEFAULT_NAME_LEN];
                 snprintf(sequence_node_name, DEFAULT_NAME_LEN, "%d", seq_idx++);
@@ -391,11 +427,11 @@ ConfYamlParse(yaml_parser_t *parser, ConfNode *parent, int inseq, int rlevel)
                 }
                 seq_node->is_seq = 1;
                 TAILQ_INSERT_TAIL(&node->head, seq_node, next);
-                if (ConfYamlParse(parser, seq_node, 0, rlevel) != 0)
+                if (ConfYamlParse(parser, seq_node, 0, rlevel, 0) != 0)
                     goto fail;
             }
             else {
-                if (ConfYamlParse(parser, node, inseq, rlevel) != 0)
+                if (ConfYamlParse(parser, node, inseq, rlevel, 0) != 0)
                     goto fail;
             }
             state = CONF_KEY;
@@ -444,15 +480,16 @@ ConfYamlLoadFile(const char *filename)
     ConfNode *root = ConfGetRootNode();
 
     if (yaml_parser_initialize(&parser) != 1) {
-        SCLogError(SC_ERR_FATAL, "failed to initialize yaml parser.");
+        SCLogError("failed to initialize yaml parser.");
         return -1;
     }
 
     struct stat stat_buf;
     if (stat(filename, &stat_buf) == 0) {
         if (stat_buf.st_mode & S_IFDIR) {
-            SCLogError(SC_ERR_FATAL, "yaml argument is not a file but a directory: %s. "
-                    "Please specify the yaml file in your -c option.", filename);
+            SCLogError("yaml argument is not a file but a directory: %s. "
+                       "Please specify the yaml file in your -c option.",
+                    filename);
             yaml_parser_delete(&parser);
             return -1;
         }
@@ -461,8 +498,7 @@ ConfYamlLoadFile(const char *filename)
     // coverity[toctou : FALSE]
     infile = fopen(filename, "r");
     if (infile == NULL) {
-        SCLogError(SC_ERR_FATAL, "failed to open file: %s: %s", filename,
-            strerror(errno));
+        SCLogError("failed to open file: %s: %s", filename, strerror(errno));
         yaml_parser_delete(&parser);
         return -1;
     }
@@ -472,7 +508,7 @@ ConfYamlLoadFile(const char *filename)
     }
 
     yaml_parser_set_input_file(&parser, infile);
-    ret = ConfYamlParse(&parser, root, 0, 0);
+    ret = ConfYamlParse(&parser, root, 0, 0, 0);
     yaml_parser_delete(&parser);
     fclose(infile);
 
@@ -494,7 +530,7 @@ ConfYamlLoadString(const char *string, size_t len)
         exit(EXIT_FAILURE);
     }
     yaml_parser_set_input_string(&parser, (const unsigned char *)string, len);
-    ret = ConfYamlParse(&parser, root, 0, 0);
+    ret = ConfYamlParse(&parser, root, 0, 0, 0);
     yaml_parser_delete(&parser);
 
     return ret;
@@ -522,7 +558,7 @@ ConfYamlLoadFileWithPrefix(const char *filename, const char *prefix)
     ConfNode *root = ConfGetNode(prefix);
 
     if (yaml_parser_initialize(&parser) != 1) {
-        SCLogError(SC_ERR_FATAL, "failed to initialize yaml parser.");
+        SCLogError("failed to initialize yaml parser.");
         return -1;
     }
 
@@ -530,8 +566,9 @@ ConfYamlLoadFileWithPrefix(const char *filename, const char *prefix)
     /* coverity[toctou] */
     if (stat(filename, &stat_buf) == 0) {
         if (stat_buf.st_mode & S_IFDIR) {
-            SCLogError(SC_ERR_FATAL, "yaml argument is not a file but a directory: %s. "
-                    "Please specify the yaml file in your -c option.", filename);
+            SCLogError("yaml argument is not a file but a directory: %s. "
+                       "Please specify the yaml file in your -c option.",
+                    filename);
             return -1;
         }
     }
@@ -539,8 +576,7 @@ ConfYamlLoadFileWithPrefix(const char *filename, const char *prefix)
     /* coverity[toctou] */
     infile = fopen(filename, "r");
     if (infile == NULL) {
-        SCLogError(SC_ERR_FATAL, "failed to open file: %s: %s", filename,
-            strerror(errno));
+        SCLogError("failed to open file: %s: %s", filename, strerror(errno));
         yaml_parser_delete(&parser);
         return -1;
     }
@@ -560,7 +596,7 @@ ConfYamlLoadFileWithPrefix(const char *filename, const char *prefix)
         }
     }
     yaml_parser_set_input_file(&parser, infile);
-    ret = ConfYamlParse(&parser, root, 0, 0);
+    ret = ConfYamlParse(&parser, root, 0, 0, 0);
     yaml_parser_delete(&parser);
     fclose(infile);
 
@@ -589,39 +625,28 @@ default-log-dir: /tmp\n\
 
     ConfNode *node;
     node = ConfGetNode("rule-files");
-    if (node == NULL)
-        return 0;
-    if (!ConfNodeIsSequence(node))
-        return 0;
-    if (TAILQ_EMPTY(&node->head))
-        return 0;
+    FAIL_IF_NULL(node);
+    FAIL_IF_NOT(ConfNodeIsSequence(node));
+    FAIL_IF(TAILQ_EMPTY(&node->head));
     int i = 0;
     ConfNode *filename;
     TAILQ_FOREACH(filename, &node->head, next) {
         if (i == 0) {
-            if (strcmp(filename->val, "netbios.rules") != 0)
-                return 0;
-            if (ConfNodeIsSequence(filename))
-                return 0;
-            if (filename->is_seq != 0)
-                return 0;
+            FAIL_IF(strcmp(filename->val, "netbios.rules") != 0);
+            FAIL_IF(ConfNodeIsSequence(filename));
+            FAIL_IF(filename->is_seq != 0);
         }
         else if (i == 1) {
-            if (strcmp(filename->val, "x11.rules") != 0)
-                return 0;
-            if (ConfNodeIsSequence(filename))
-                return 0;
-        }
-        else {
-            return 0;
+            FAIL_IF(strcmp(filename->val, "x11.rules") != 0);
+            FAIL_IF(ConfNodeIsSequence(filename));
         }
+        FAIL_IF(i > 1);
         i++;
     }
 
     ConfDeInit();
     ConfRestoreContextBackup();
-
-    return 1;
+    PASS;
 }
 
 static int
@@ -646,57 +671,45 @@ logging:\n\
 
     ConfNode *outputs;
     outputs = ConfGetNode("logging.output");
-    if (outputs == NULL)
-        return 0;
+    FAIL_IF_NULL(outputs);
 
     ConfNode *output;
     ConfNode *output_param;
 
     output = TAILQ_FIRST(&outputs->head);
-    if (output == NULL)
-        return 0;
-    if (strcmp(output->name, "0") != 0)
-        return 0;
+    FAIL_IF_NULL(output);
+    FAIL_IF(strcmp(output->name, "0") != 0);
+
     output_param = TAILQ_FIRST(&output->head);
-    if (output_param == NULL)
-        return 0;
-    if (strcmp(output_param->name, "interface") != 0)
-        return 0;
-    if (strcmp(output_param->val, "console") != 0)
-        return 0;
+    FAIL_IF_NULL(output_param);
+    FAIL_IF(strcmp(output_param->name, "interface") != 0);
+    FAIL_IF(strcmp(output_param->val, "console") != 0);
+
     output_param = TAILQ_NEXT(output_param, next);
-    if (strcmp(output_param->name, "log-level") != 0)
-        return 0;
-    if (strcmp(output_param->val, "error") != 0)
-        return 0;
+    FAIL_IF(strcmp(output_param->name, "log-level") != 0);
+    FAIL_IF(strcmp(output_param->val, "error") != 0);
 
     output = TAILQ_NEXT(output, next);
-    if (output == NULL)
-        return 0;
-    if (strcmp(output->name, "1") != 0)
-        return 0;
+    FAIL_IF_NULL(output);
+    FAIL_IF(strcmp(output->name, "1") != 0);
+
     output_param = TAILQ_FIRST(&output->head);
-    if (output_param == NULL)
-        return 0;
-    if (strcmp(output_param->name, "interface") != 0)
-        return 0;
-    if (strcmp(output_param->val, "syslog") != 0)
-        return 0;
+    FAIL_IF_NULL(output_param);
+    FAIL_IF(strcmp(output_param->name, "interface") != 0);
+    FAIL_IF(strcmp(output_param->val, "syslog") != 0);
+
     output_param = TAILQ_NEXT(output_param, next);
-    if (strcmp(output_param->name, "facility") != 0)
-        return 0;
-    if (strcmp(output_param->val, "local4") != 0)
-        return 0;
+    FAIL_IF(strcmp(output_param->name, "facility") != 0);
+    FAIL_IF(strcmp(output_param->val, "local4") != 0);
+
     output_param = TAILQ_NEXT(output_param, next);
-    if (strcmp(output_param->name, "log-level") != 0)
-        return 0;
-    if (strcmp(output_param->val, "info") != 0)
-        return 0;
+    FAIL_IF(strcmp(output_param->name, "log-level") != 0);
+    FAIL_IF(strcmp(output_param->val, "info") != 0);
 
     ConfDeInit();
     ConfRestoreContextBackup();
 
-    return 1;
+    PASS;
 }
 
 /**
@@ -708,13 +721,12 @@ ConfYamlNonYamlFileTest(void)
     ConfCreateContextBackup();
     ConfInit();
 
-    if (ConfYamlLoadFile("/etc/passwd") != -1)
-        return 0;
+    FAIL_IF(ConfYamlLoadFile("/etc/passwd") != -1);
 
     ConfDeInit();
     ConfRestoreContextBackup();
 
-    return 1;
+    PASS;
 }
 
 static int
@@ -735,13 +747,12 @@ logging:\n\
     ConfCreateContextBackup();
     ConfInit();
 
-    if (ConfYamlLoadString(input, strlen(input)) != -1)
-        return 0;
+    FAIL_IF(ConfYamlLoadString(input, strlen(input)) != -1);
 
     ConfDeInit();
     ConfRestoreContextBackup();
 
-    return 1;
+    PASS;
 }
 
 static int
@@ -771,42 +782,34 @@ libhtp:\n\
     ConfCreateContextBackup();
     ConfInit();
 
-    if (ConfYamlLoadString(input, strlen(input)) != 0)
-        return 0;
+    FAIL_IF(ConfYamlLoadString(input, strlen(input)) != 0);
 
     ConfNode *outputs;
     outputs = ConfGetNode("libhtp.server-config");
-    if (outputs == NULL)
-        return 0;
+    FAIL_IF_NULL(outputs);
 
     ConfNode *node;
 
     node = TAILQ_FIRST(&outputs->head);
-    if (node == NULL)
-        return 0;
-    if (strcmp(node->name, "0") != 0)
-        return 0;
+    FAIL_IF_NULL(node);
+    FAIL_IF(strcmp(node->name, "0") != 0);
+
     node = TAILQ_FIRST(&node->head);
-    if (node == NULL)
-        return 0;
-    if (strcmp(node->name, "apache-php") != 0)
-        return 0;
+    FAIL_IF_NULL(node);
+    FAIL_IF(strcmp(node->name, "apache-php") != 0);
 
     node = ConfNodeLookupChild(node, "address");
-    if (node == NULL)
-        return 0;
+    FAIL_IF_NULL(node);
+
     node = TAILQ_FIRST(&node->head);
-    if (node == NULL)
-        return 0;
-    if (strcmp(node->name, "0") != 0)
-        return 0;
-    if (strcmp(node->val, "192.168.1.0/24") != 0)
-        return 0;
+    FAIL_IF_NULL(node);
+    FAIL_IF(strcmp(node->name, "0") != 0);
+    FAIL_IF(strcmp(node->val, "192.168.1.0/24") != 0);
 
     ConfDeInit();
     ConfRestoreContextBackup();
 
-    return 1;
+    PASS;
 }
 
 /**
@@ -815,7 +818,6 @@ libhtp:\n\
 static int
 ConfYamlFileIncludeTest(void)
 {
-    int ret = 0;
     FILE *config_file;
 
     const char config_filename[] = "ConfYamlFileIncludeTest-config.yaml";
@@ -839,21 +841,12 @@ ConfYamlFileIncludeTest(void)
     ConfInit();
 
     /* Write out the test files. */
-    if ((config_file = fopen(config_filename, "w")) == NULL) {
-        goto cleanup;
-    }
-    if (fwrite(config_file_contents, strlen(config_file_contents), 1,
-            config_file) != 1) {
-        goto cleanup;
-    }
+    FAIL_IF_NULL((config_file = fopen(config_filename, "w")));
+    FAIL_IF(fwrite(config_file_contents, strlen(config_file_contents), 1, config_file) != 1);
     fclose(config_file);
-    if ((config_file = fopen(include_filename, "w")) == NULL) {
-        goto cleanup;
-    }
-    if (fwrite(include_file_contents, strlen(include_file_contents), 1,
-            config_file) != 1) {
-        goto cleanup;
-    }
+
+    FAIL_IF_NULL((config_file = fopen(include_filename, "w")));
+    FAIL_IF(fwrite(include_file_contents, strlen(include_file_contents), 1, config_file) != 1);
     fclose(config_file);
 
     /* Reset conf_dirname. */
@@ -862,45 +855,35 @@ ConfYamlFileIncludeTest(void)
         conf_dirname = NULL;
     }
 
-    if (ConfYamlLoadFile("ConfYamlFileIncludeTest-config.yaml") != 0)
-        goto cleanup;
+    FAIL_IF(ConfYamlLoadFile("ConfYamlFileIncludeTest-config.yaml") != 0);
 
     /* Check values that should have been loaded into the root of the
      * configuration. */
     ConfNode *node;
     node = ConfGetNode("host-mode");
-    if (node == NULL)
-        goto cleanup;
-    if (strcmp(node->val, "auto") != 0)
-        goto cleanup;
+    FAIL_IF_NULL(node);
+    FAIL_IF(strcmp(node->val, "auto") != 0);
+
     node = ConfGetNode("unix-command.enabled");
-    if (node == NULL)
-        goto cleanup;
-    if (strcmp(node->val, "no") != 0)
-        goto cleanup;
+    FAIL_IF_NULL(node);
+    FAIL_IF(strcmp(node->val, "no") != 0);
 
     /* Check for values that were included under a mapping. */
     node = ConfGetNode("mapping.host-mode");
-    if (node == NULL)
-        goto cleanup;
-    if (strcmp(node->val, "auto") != 0)
-        goto cleanup;
+    FAIL_IF_NULL(node);
+    FAIL_IF(strcmp(node->val, "auto") != 0);
+
     node = ConfGetNode("mapping.unix-command.enabled");
-    if (node == NULL)
-        goto cleanup;
-    if (strcmp(node->val, "no") != 0)
-        goto cleanup;
+    FAIL_IF_NULL(node);
+    FAIL_IF(strcmp(node->val, "no") != 0);
 
     ConfDeInit();
     ConfRestoreContextBackup();
 
-    ret = 1;
-
-cleanup:
     unlink(config_filename);
     unlink(include_filename);
 
-    return ret;
+    PASS;
 }
 
 /**
@@ -910,43 +893,58 @@ cleanup:
 static int
 ConfYamlOverrideTest(void)
 {
-    char config[] =
-        "%YAML 1.1\n"
-        "---\n"
-        "some-log-dir: /var/log\n"
-        "some-log-dir: /tmp\n"
-        "\n"
-        "parent:\n"
-        "  child0:\n"
-        "    key: value\n"
-        "parent:\n"
-        "  child1:\n"
-        "    key: value\n"
-        ;
+    char config[] = "%YAML 1.1\n"
+                    "---\n"
+                    "some-log-dir: /var/log\n"
+                    "some-log-dir: /tmp\n"
+                    "\n"
+                    "parent:\n"
+                    "  child0:\n"
+                    "    key: value\n"
+                    "parent:\n"
+                    "  child1:\n"
+                    "    key: value\n"
+                    "vars:\n"
+                    "  address-groups:\n"
+                    "    HOME_NET: \"[192.168.0.0/16,10.0.0.0/8,172.16.0.0/12]\"\n"
+                    "    EXTERNAL_NET: any\n"
+                    "vars.address-groups.HOME_NET: \"10.10.10.10/32\"\n";
     const char *value;
 
     ConfCreateContextBackup();
     ConfInit();
 
-    if (ConfYamlLoadString(config, strlen(config)) != 0)
-        return 0;
-    if (!ConfGet("some-log-dir", &value))
-        return 0;
-    if (strcmp(value, "/tmp") != 0)
-        return 0;
+    FAIL_IF(ConfYamlLoadString(config, strlen(config)) != 0);
+    FAIL_IF_NOT(ConfGet("some-log-dir", &value));
+    FAIL_IF(strcmp(value, "/tmp") != 0);
 
     /* Test that parent.child0 does not exist, but child1 does. */
-    if (ConfGetNode("parent.child0") != NULL)
-        return 0;
-    if (!ConfGet("parent.child1.key", &value))
-        return 0;
-    if (strcmp(value, "value") != 0)
-        return 0;
+    FAIL_IF_NOT_NULL(ConfGetNode("parent.child0"));
+    FAIL_IF_NOT(ConfGet("parent.child1.key", &value));
+    FAIL_IF(strcmp(value, "value") != 0);
+
+    /* First check that vars.address-groups.EXTERNAL_NET has the
+     * expected parent of vars.address-groups and save this
+     * pointer. We want to make sure that the overrided value has the
+     * same parent later on. */
+    ConfNode *vars_address_groups = ConfGetNode("vars.address-groups");
+    FAIL_IF_NULL(vars_address_groups);
+    ConfNode *vars_address_groups_external_net = ConfGetNode("vars.address-groups.EXTERNAL_NET");
+    FAIL_IF_NULL(vars_address_groups_external_net);
+    FAIL_IF_NOT(vars_address_groups_external_net->parent == vars_address_groups);
+
+    /* Now check that HOME_NET has the overrided value. */
+    ConfNode *vars_address_groups_home_net = ConfGetNode("vars.address-groups.HOME_NET");
+    FAIL_IF_NULL(vars_address_groups_home_net);
+    FAIL_IF(strcmp(vars_address_groups_home_net->val, "10.10.10.10/32") != 0);
+
+    /* And check that it has the correct parent. */
+    FAIL_IF_NOT(vars_address_groups_home_net->parent == vars_address_groups);
 
     ConfDeInit();
     ConfRestoreContextBackup();
 
-    return 1;
+    PASS;
 }
 
 /**
@@ -965,24 +963,91 @@ ConfYamlOverrideFinalTest(void)
         "default-log-dir: /var/log\n";
 
     /* Set the log directory as if it was set on the command line. */
-    if (!ConfSetFinal("default-log-dir", "/tmp"))
-        return 0;
-    if (ConfYamlLoadString(config, strlen(config)) != 0)
-        return 0;
+    FAIL_IF_NOT(ConfSetFinal("default-log-dir", "/tmp"));
+    FAIL_IF(ConfYamlLoadString(config, strlen(config)) != 0);
 
     const char *default_log_dir;
 
-    if (!ConfGet("default-log-dir", &default_log_dir))
-        return 0;
-    if (strcmp(default_log_dir, "/tmp") != 0) {
-        fprintf(stderr, "final value was reassigned\n");
-        return 0;
-    }
+    FAIL_IF_NOT(ConfGet("default-log-dir", &default_log_dir));
+    FAIL_IF(strcmp(default_log_dir, "/tmp") != 0);
 
     ConfDeInit();
     ConfRestoreContextBackup();
 
-    return 1;
+    PASS;
+}
+
+static int ConfYamlNull(void)
+{
+    ConfCreateContextBackup();
+    ConfInit();
+
+    char config[] = "%YAML 1.1\n"
+                    "---\n"
+                    "quoted-tilde: \"~\"\n"
+                    "unquoted-tilde: ~\n"
+                    "quoted-null: \"null\"\n"
+                    "unquoted-null: null\n"
+                    "quoted-Null: \"Null\"\n"
+                    "unquoted-Null: Null\n"
+                    "quoted-NULL: \"NULL\"\n"
+                    "unquoted-NULL: NULL\n"
+                    "empty-quoted: \"\"\n"
+                    "empty-unquoted: \n"
+                    "list: [\"null\", null, \"Null\", Null, \"NULL\", NULL, \"~\", ~]\n";
+    FAIL_IF(ConfYamlLoadString(config, strlen(config)) != 0);
+
+    const char *val;
+
+    FAIL_IF_NOT(ConfGet("quoted-tilde", &val));
+    FAIL_IF_NULL(val);
+    FAIL_IF_NOT(ConfGet("unquoted-tilde", &val));
+    FAIL_IF_NOT_NULL(val);
+
+    FAIL_IF_NOT(ConfGet("quoted-null", &val));
+    FAIL_IF_NULL(val);
+    FAIL_IF_NOT(ConfGet("unquoted-null", &val));
+    FAIL_IF_NOT_NULL(val);
+
+    FAIL_IF_NOT(ConfGet("quoted-Null", &val));
+    FAIL_IF_NULL(val);
+    FAIL_IF_NOT(ConfGet("unquoted-Null", &val));
+    FAIL_IF_NOT_NULL(val);
+
+    FAIL_IF_NOT(ConfGet("quoted-NULL", &val));
+    FAIL_IF_NULL(val);
+    FAIL_IF_NOT(ConfGet("unquoted-NULL", &val));
+    FAIL_IF_NOT_NULL(val);
+
+    FAIL_IF_NOT(ConfGet("empty-quoted", &val));
+    FAIL_IF_NULL(val);
+    FAIL_IF_NOT(ConfGet("empty-unquoted", &val));
+    FAIL_IF_NOT_NULL(val);
+
+    FAIL_IF_NOT(ConfGet("list.0", &val));
+    FAIL_IF_NULL(val);
+    FAIL_IF_NOT(ConfGet("list.1", &val));
+    FAIL_IF_NOT_NULL(val);
+
+    FAIL_IF_NOT(ConfGet("list.2", &val));
+    FAIL_IF_NULL(val);
+    FAIL_IF_NOT(ConfGet("list.3", &val));
+    FAIL_IF_NOT_NULL(val);
+
+    FAIL_IF_NOT(ConfGet("list.4", &val));
+    FAIL_IF_NULL(val);
+    FAIL_IF_NOT(ConfGet("list.5", &val));
+    FAIL_IF_NOT_NULL(val);
+
+    FAIL_IF_NOT(ConfGet("list.6", &val));
+    FAIL_IF_NULL(val);
+    FAIL_IF_NOT(ConfGet("list.7", &val));
+    FAIL_IF_NOT_NULL(val);
+
+    ConfDeInit();
+    ConfRestoreContextBackup();
+
+    PASS;
 }
 
 #endif /* UNITTESTS */
@@ -1000,5 +1065,6 @@ ConfYamlRegisterTests(void)
     UtRegisterTest("ConfYamlFileIncludeTest", ConfYamlFileIncludeTest);
     UtRegisterTest("ConfYamlOverrideTest", ConfYamlOverrideTest);
     UtRegisterTest("ConfYamlOverrideFinalTest", ConfYamlOverrideFinalTest);
+    UtRegisterTest("ConfYamlNull", ConfYamlNull);
 #endif /* UNITTESTS */
 }
