@@ -32,19 +32,11 @@
  */
 
 #include "suricata-common.h"
-#include "packet-queue.h"
-#include "decode.h"
 #include "decode-ipv4.h"
-#include "decode-events.h"
+#include "decode.h"
 #include "defrag.h"
-#include "pkt-var.h"
-#include "host.h"
-
-#include "util-unittest.h"
-#include "util-debug.h"
-#include "util-optimize.h"
+#include "flow.h"
 #include "util-print.h"
-#include "util-profiling.h"
 
 /* Generic validation
  *
@@ -66,7 +58,8 @@ static int IPV4OptValidateGeneric(Packet *p, const IPV4Opt *o)
             break;
         /* See: RFC 1108 */
         case IPV4_OPT_SEC:
-            if (o->len != IPV4_OPT_SEC_LEN) {
+        case IPV4_OPT_ESEC:
+            if (unlikely(o->len < IPV4_OPT_SEC_MIN)) {
                 ENGINE_SET_INVALID_EVENT(p, IPV4_OPT_INVALID_LEN);
                 return -1;
             }
@@ -159,7 +152,7 @@ static int IPV4OptValidateTimestamp(Packet *p, const IPV4Opt *o)
         ENGINE_SET_INVALID_EVENT(p, IPV4_OPT_MALFORMED);
         return -1;
     }
-    flag = *(o->data + 3) & 0x00ff;
+    flag = *(o->data + 1) & 0x0f;
 
     /* A flag of 1|3 means we have both the ip+ts in each record */
     rec_size = ((flag == 1) || (flag == 3)) ? 8 : 4;
@@ -186,7 +179,7 @@ static int IPV4OptValidateTimestamp(Packet *p, const IPV4Opt *o)
 static int IPV4OptValidateCIPSO(Packet *p, const IPV4Opt *o)
 {
 //    uint32_t doi;
-    uint8_t *tag;
+    const uint8_t *tag;
     uint16_t len;
 
     /* Check length */
@@ -207,7 +200,7 @@ static int IPV4OptValidateCIPSO(Packet *p, const IPV4Opt *o)
 
 #if 0
     /* Domain of Interest (DOI) of 0 is reserved and thus invalid */
-    /** \todo Aparently a DOI of zero is fine in practice - verify. */
+    /** \todo Apparently a DOI of zero is fine in practice - verify. */
     if (doi == 0) {
         ENGINE_SET_EVENT(p,IPV4_OPT_MALFORMED);
         return -1;
@@ -290,6 +283,7 @@ typedef struct IPV4Options_ {
     IPV4Opt o_ts;
     IPV4Opt o_sec;
     IPV4Opt o_lsrr;
+    IPV4Opt o_esec;
     IPV4Opt o_cipso;
     IPV4Opt o_sid;
     IPV4Opt o_ssrr;
@@ -299,7 +293,7 @@ typedef struct IPV4Options_ {
 /**
  * Decode/Validate IPv4 Options.
  */
-static int DecodeIPV4Options(Packet *p, uint8_t *pkt, uint16_t len, IPV4Options *opts)
+static int DecodeIPV4Options(Packet *p, const uint8_t *pkt, uint16_t len, IPV4Options *opts)
 {
     uint16_t plen = len;
 
@@ -329,12 +323,12 @@ static int DecodeIPV4Options(Packet *p, uint8_t *pkt, uint16_t len, IPV4Options
         /* single byte options */
         if (*pkt == IPV4_OPT_EOL) {
             /** \todo What if more data exist after EOL (possible covert channel or data leakage)? */
-            SCLogDebug("IPV4OPT %" PRIu16 " len 1 @ %" PRIu16 "/%" PRIu16 "",
+            SCLogDebug("IPV4OPT %" PRIu8 " len 1 @ %d/%d",
                    *pkt, (len - plen), (len - 1));
             p->ip4vars.opts_set |= IPV4_OPT_FLAG_EOL;
             break;
         } else if (*pkt == IPV4_OPT_NOP) {
-            SCLogDebug("IPV4OPT %" PRIu16 " len 1 @ %" PRIu16 "/%" PRIu16 "",
+            SCLogDebug("IPV4OPT %" PRIu8 " len 1 @ %d/%d",
                    *pkt, (len - plen), (len - 1));
             pkt++;
             plen--;
@@ -374,100 +368,91 @@ static int DecodeIPV4Options(Packet *p, uint8_t *pkt, uint16_t len, IPV4Options
                     if (opts->o_ts.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateTimestamp(p, &opt)) {
-                        return -1;
+                    } else if (IPV4OptValidateTimestamp(p, &opt) == 0) {
+                        opts->o_ts = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_TS;
                     }
-                    opts->o_ts = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_TS;
                     break;
                 case IPV4_OPT_RR:
                     if (opts->o_rr.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateRoute(p, &opt) != 0) {
-                        return -1;
+                    } else if (IPV4OptValidateRoute(p, &opt) == 0) {
+                        opts->o_rr = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_RR;
                     }
-                    opts->o_rr = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_RR;
                     break;
                 case IPV4_OPT_QS:
                     if (opts->o_qs.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateGeneric(p, &opt)) {
-                        return -1;
+                    } else if (IPV4OptValidateGeneric(p, &opt) == 0) {
+                        opts->o_qs = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_QS;
                     }
-                    opts->o_qs = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_QS;
                     break;
                 case IPV4_OPT_SEC:
                     if (opts->o_sec.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateGeneric(p, &opt)) {
-                        return -1;
+                    } else if (IPV4OptValidateGeneric(p, &opt) == 0) {
+                        opts->o_sec = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_SEC;
                     }
-                    opts->o_sec = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_SEC;
                     break;
                 case IPV4_OPT_LSRR:
                     if (opts->o_lsrr.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateRoute(p, &opt) != 0) {
-                        return -1;
+                    } else if (IPV4OptValidateRoute(p, &opt) == 0) {
+                        opts->o_lsrr = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_LSRR;
+                    }
+                    break;
+                case IPV4_OPT_ESEC:
+                    if (opts->o_esec.type != 0) {
+                        ENGINE_SET_EVENT(p, IPV4_OPT_DUPLICATE);
+                        /* Warn - we can keep going */
+                    } else if (IPV4OptValidateGeneric(p, &opt) == 0) {
+                        opts->o_esec = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_ESEC;
                     }
-                    opts->o_lsrr = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_LSRR;
                     break;
                 case IPV4_OPT_CIPSO:
                     if (opts->o_cipso.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateCIPSO(p, &opt) != 0) {
-                        return -1;
+                    } else if (IPV4OptValidateCIPSO(p, &opt) == 0) {
+                        opts->o_cipso = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_CIPSO;
                     }
-                    opts->o_cipso = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_CIPSO;
                     break;
                 case IPV4_OPT_SID:
                     if (opts->o_sid.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateGeneric(p, &opt)) {
-                        return -1;
+                    } else if (IPV4OptValidateGeneric(p, &opt) == 0) {
+                        opts->o_sid = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_SID;
                     }
-                    opts->o_sid = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_SID;
                     break;
                 case IPV4_OPT_SSRR:
                     if (opts->o_ssrr.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateRoute(p, &opt) != 0) {
-                        return -1;
+                    } else if (IPV4OptValidateRoute(p, &opt) == 0) {
+                        opts->o_ssrr = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_SSRR;
                     }
-                    opts->o_ssrr = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_SSRR;
                     break;
                 case IPV4_OPT_RTRALT:
                     if (opts->o_rtralt.type != 0) {
                         ENGINE_SET_EVENT(p,IPV4_OPT_DUPLICATE);
                         /* Warn - we can keep going */
-                        break;
-                    } else if (IPV4OptValidateGeneric(p, &opt)) {
-                        return -1;
+                    } else if (IPV4OptValidateGeneric(p, &opt) == 0) {
+                        opts->o_rtralt = opt;
+                        p->ip4vars.opts_set |= IPV4_OPT_FLAG_RTRALT;
                     }
-                    opts->o_rtralt = opt;
-                    p->ip4vars.opts_set |= IPV4_OPT_FLAG_RTRALT;
                     break;
                 default:
                     SCLogDebug("IPV4OPT <unknown> (%" PRIu8 ") len %" PRIu8,
@@ -485,7 +470,7 @@ static int DecodeIPV4Options(Packet *p, uint8_t *pkt, uint16_t len, IPV4Options
     return 0;
 }
 
-static int DecodeIPV4Packet(Packet *p, uint8_t *pkt, uint16_t len)
+static int DecodeIPV4Packet(Packet *p, const uint8_t *pkt, uint16_t len)
 {
     if (unlikely(len < IPV4_HEADER_LEN)) {
         ENGINE_SET_INVALID_EVENT(p, IPV4_PKT_TOO_SMALL);
@@ -493,7 +478,7 @@ static int DecodeIPV4Packet(Packet *p, uint8_t *pkt, uint16_t len)
     }
 
     if (unlikely(IP_GET_RAW_VER(pkt) != 4)) {
-        SCLogDebug("wrong ip version %" PRIu8 "",IP_GET_RAW_VER(pkt));
+        SCLogDebug("wrong ip version %d",IP_GET_RAW_VER(pkt));
         ENGINE_SET_INVALID_EVENT(p, IPV4_WRONG_IP_VER);
         return -1;
     }
@@ -524,31 +509,37 @@ static int DecodeIPV4Packet(Packet *p, uint8_t *pkt, uint16_t len)
     if (ip_opt_len > 0) {
         IPV4Options opts;
         memset(&opts, 0x00, sizeof(opts));
-        DecodeIPV4Options(p, pkt + IPV4_HEADER_LEN, ip_opt_len, &opts);
+        if (DecodeIPV4Options(p, pkt + IPV4_HEADER_LEN, ip_opt_len, &opts) < 0) {
+            return -1;
+        }
     }
 
     return 0;
 }
 
-int DecodeIPV4(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, uint16_t len, PacketQueue *pq)
+int DecodeIPV4(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p,
+        const uint8_t *pkt, uint16_t len)
 {
     StatsIncr(tv, dtv->counter_ipv4);
 
     SCLogDebug("pkt %p len %"PRIu16"", pkt, len);
 
+    if (!PacketIncreaseCheckLayers(p)) {
+        return TM_ECODE_FAILED;
+    }
     /* do the actual decoding */
     if (unlikely(DecodeIPV4Packet (p, pkt, len) < 0)) {
         SCLogDebug("decoding IPv4 packet failed");
-        p->ip4h = NULL;
+        CLEAR_IPV4_PACKET((p));
         return TM_ECODE_FAILED;
     }
     p->proto = IPV4_GET_IPPROTO(p);
 
     /* If a fragment, pass off for re-assembly. */
     if (unlikely(IPV4_GET_IPOFFSET(p) > 0 || IPV4_GET_MF(p) == 1)) {
-        Packet *rp = Defrag(tv, dtv, p, pq);
+        Packet *rp = Defrag(tv, dtv, p);
         if (rp != NULL) {
-            PacketEnqueue(pq, rp);
+            PacketEnqueueNoLock(&tv->decode_pq, rp);
         }
         p->flags |= PKT_IS_FRAGMENT;
         return TM_ECODE_OK;
@@ -572,44 +563,47 @@ int DecodeIPV4(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, u
     switch (IPV4_GET_IPPROTO(p)) {
         case IPPROTO_TCP:
             DecodeTCP(tv, dtv, p, pkt + IPV4_GET_HLEN(p),
-                      IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p), pq);
+                      IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p));
             break;
         case IPPROTO_UDP:
             DecodeUDP(tv, dtv, p, pkt + IPV4_GET_HLEN(p),
-                      IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p), pq);
+                      IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p));
             break;
         case IPPROTO_ICMP:
             DecodeICMPV4(tv, dtv, p, pkt + IPV4_GET_HLEN(p),
-                         IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p), pq);
+                         IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p));
             break;
         case IPPROTO_GRE:
             DecodeGRE(tv, dtv, p, pkt + IPV4_GET_HLEN(p),
-                      IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p), pq);
+                      IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p));
             break;
         case IPPROTO_SCTP:
             DecodeSCTP(tv, dtv, p, pkt + IPV4_GET_HLEN(p),
-                      IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p), pq);
+                      IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p));
             break;
+
+        case IPPROTO_ESP:
+            DecodeESP(tv, dtv, p, pkt + IPV4_GET_HLEN(p), IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p));
+            break;
+
         case IPPROTO_IPV6:
             {
-                if (pq != NULL) {
-                    /* spawn off tunnel packet */
-                    Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + IPV4_GET_HLEN(p),
-                            IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p),
-                            DECODE_TUNNEL_IPV6, pq);
-                    if (tp != NULL) {
-                        PKT_SET_SRC(tp, PKT_SRC_DECODER_IPV4);
-                        PacketEnqueue(pq,tp);
-                    }
-                    FlowSetupPacket(p);
+                /* spawn off tunnel packet */
+                Packet *tp = PacketTunnelPktSetup(tv, dtv, p, pkt + IPV4_GET_HLEN(p),
+                        IPV4_GET_IPLEN(p) - IPV4_GET_HLEN(p),
+                        DECODE_TUNNEL_IPV6);
+                if (tp != NULL) {
+                    PKT_SET_SRC(tp, PKT_SRC_DECODER_IPV4);
+                    PacketEnqueueNoLock(&tv->decode_pq,tp);
                 }
+                FlowSetupPacket(p);
                 break;
             }
         case IPPROTO_IP:
             /* check PPP VJ uncompressed packets and decode tcp dummy */
             if(p->ppph != NULL && SCNtohs(p->ppph->protocol) == PPP_VJ_UCOMP)    {
                 DecodeTCP(tv, dtv, p, pkt + IPV4_GET_HLEN(p),
-                          IPV4_GET_IPLEN(p) -  IPV4_GET_HLEN(p), pq);
+                          IPV4_GET_IPLEN(p) -  IPV4_GET_HLEN(p));
             }
             break;
         case IPPROTO_ICMPV6:
@@ -622,6 +616,7 @@ int DecodeIPV4(ThreadVars *tv, DecodeThreadVars *dtv, Packet *p, uint8_t *pkt, u
 
 /* UNITTESTS */
 #ifdef UNITTESTS
+#include "packet.h"
 
 /** \test IPV4 with no options. */
 static int DecodeIPV4OptionsNONETest01(void)
@@ -632,8 +627,8 @@ static int DecodeIPV4OptionsNONETest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
 
     SCFree(p);
     PASS;
@@ -649,8 +644,8 @@ static int DecodeIPV4OptionsEOLTest01(void)
     FAIL_IF(unlikely(p == NULL));
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF (rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     SCFree(p);
     PASS;
 }
@@ -665,8 +660,8 @@ static int DecodeIPV4OptionsNOPTest01(void)
     FAIL_IF(unlikely(p == NULL));
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF (rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     SCFree(p);
     PASS;
 }
@@ -686,8 +681,8 @@ static int DecodeIPV4OptionsRRTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_rr.type != IPV4_OPT_RR);
     SCFree(p);
     PASS;
@@ -708,8 +703,8 @@ static int DecodeIPV4OptionsRRTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    FAIL_IF(DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts) != -1);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_rr.type != 0);
     SCFree(p);
     PASS;
@@ -730,8 +725,8 @@ static int DecodeIPV4OptionsRRTest03(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_rr.type != 0);
     SCFree(p);
     PASS;
@@ -752,8 +747,8 @@ static int DecodeIPV4OptionsRRTest04(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_rr.type != 0);
     SCFree(p);
     PASS;
@@ -770,8 +765,8 @@ static int DecodeIPV4OptionsQSTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_qs.type != IPV4_OPT_QS);
     SCFree(p);
     PASS;
@@ -788,8 +783,8 @@ static int DecodeIPV4OptionsQSTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_qs.type != 0);
     SCFree(p);
     PASS;
@@ -810,8 +805,8 @@ static int DecodeIPV4OptionsTSTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_ts.type != IPV4_OPT_TS);
     SCFree(p);
     PASS;
@@ -832,8 +827,8 @@ static int DecodeIPV4OptionsTSTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ts.type != 0);
     SCFree(p);
     PASS;
@@ -854,8 +849,8 @@ static int DecodeIPV4OptionsTSTest03(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ts.type != 0);
     SCFree(p);
     PASS;
@@ -876,8 +871,8 @@ static int DecodeIPV4OptionsTSTest04(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ts.type != 0);
     SCFree(p);
     PASS;
@@ -895,8 +890,8 @@ static int DecodeIPV4OptionsSECTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_sec.type != IPV4_OPT_SEC);
     SCFree(p);
     PASS;
@@ -905,22 +900,54 @@ static int DecodeIPV4OptionsSECTest01(void)
 /** \test IPV4 with SEC option (invalid length). */
 static int DecodeIPV4OptionsSECTest02(void)
 {
-    uint8_t raw_opts[] = {
-        IPV4_OPT_SEC, 0x0a, 0xf1, 0x35, 0x00, 0x00, 0x00, 0x00,
-        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
-    };
+    uint8_t raw_opts[] = { IPV4_OPT_SEC, 0x02, 0xf1, 0x35, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+        0x00, 0x00, 0x00, 0x00, 0x00 };
     Packet *p = PacketGetFromAlloc();
     FAIL_IF(unlikely(p == NULL));
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_sec.type != 0);
     SCFree(p);
     PASS;
 }
 
+/** \test IPV4 with ESEC option. */
+static int DecodeIPV4OptionsESECTest01(void)
+{
+    uint8_t raw_opts[] = { IPV4_OPT_ESEC, 0x0b, 0xf1, 0x35, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+    Packet *p = PacketGetFromAlloc();
+    FAIL_IF(unlikely(p == NULL));
+
+    IPV4Options opts;
+    memset(&opts, 0x00, sizeof(opts));
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
+    FAIL_IF(opts.o_esec.type != IPV4_OPT_ESEC);
+    SCFree(p);
+    PASS;
+}
+
+/** \test IPV4 with ESEC option (invalid length). */
+static int DecodeIPV4OptionsESECTest02(void)
+{
+    uint8_t raw_opts[] = { IPV4_OPT_ESEC, 0x02, 0xf1, 0x35, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+        0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+    Packet *p = PacketGetFromAlloc();
+    FAIL_IF(unlikely(p == NULL));
+
+    IPV4Options opts;
+    memset(&opts, 0x00, sizeof(opts));
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
+    FAIL_IF(opts.o_esec.type != 0);
+    SCFree(p);
+    PASS;
+}
+
 /** \test IPV4 with LSRR option. */
 static int DecodeIPV4OptionsLSRRTest01(void)
 {
@@ -936,8 +963,8 @@ static int DecodeIPV4OptionsLSRRTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_lsrr.type != IPV4_OPT_LSRR);
     SCFree(p);
     PASS;
@@ -958,8 +985,8 @@ static int DecodeIPV4OptionsLSRRTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_lsrr.type != 0);
     SCFree(p);
     PASS;
@@ -980,8 +1007,8 @@ static int DecodeIPV4OptionsLSRRTest03(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_lsrr.type != 0);
     SCFree(p);
     PASS;
@@ -1002,8 +1029,8 @@ static int DecodeIPV4OptionsLSRRTest04(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_lsrr.type != 0);
     SCFree(p);
     PASS;
@@ -1022,8 +1049,8 @@ static int DecodeIPV4OptionsCIPSOTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_cipso.type != IPV4_OPT_CIPSO);
     SCFree(p);
     PASS;
@@ -1040,8 +1067,8 @@ static int DecodeIPV4OptionsSIDTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_sid.type != IPV4_OPT_SID);
     SCFree(p);
     PASS;
@@ -1058,8 +1085,8 @@ static int DecodeIPV4OptionsSIDTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_sid.type != 0);
     SCFree(p);
     PASS;
@@ -1080,8 +1107,8 @@ static int DecodeIPV4OptionsSSRRTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_ssrr.type != IPV4_OPT_SSRR);
     SCFree(p);
     PASS;
@@ -1102,8 +1129,8 @@ static int DecodeIPV4OptionsSSRRTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ssrr.type != 0);
     SCFree(p);
     PASS;
@@ -1124,8 +1151,8 @@ static int DecodeIPV4OptionsSSRRTest03(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ssrr.type != 0);
     SCFree(p);
     PASS;
@@ -1146,8 +1173,8 @@ static int DecodeIPV4OptionsSSRRTest04(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_ssrr.type != 0);
     SCFree(p);
     PASS;
@@ -1164,8 +1191,8 @@ static int DecodeIPV4OptionsRTRALTTest01(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc != 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF(p->flags & PKT_IS_INVALID);
     FAIL_IF(opts.o_rtralt.type != IPV4_OPT_RTRALT);
     SCFree(p);
     PASS;
@@ -1182,8 +1209,8 @@ static int DecodeIPV4OptionsRTRALTTest02(void)
 
     IPV4Options opts;
     memset(&opts, 0x00, sizeof(opts));
-    int rc = DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
-    FAIL_IF(rc == 0);
+    DecodeIPV4Options(p, raw_opts, sizeof(raw_opts), &opts);
+    FAIL_IF((p->flags & PKT_IS_INVALID) == 0);
     FAIL_IF(opts.o_rtralt.type != 0);
     SCFree(p);
     PASS;
@@ -1262,45 +1289,43 @@ static int DecodeIPV4DefragTest01(void)
         return 0;
     ThreadVars tv;
     DecodeThreadVars dtv;
-    PacketQueue pq;
     int result = 1;
 
     memset(&tv, 0, sizeof(ThreadVars));
     memset(&dtv, 0, sizeof(DecodeThreadVars));
-    memset(&pq, 0, sizeof(PacketQueue));
 
     FlowInitConfig(FLOW_QUIET);
     DefragInit();
 
     PacketCopyData(p, pkt1, sizeof(pkt1));
     DecodeIPV4(&tv, &dtv, p, GET_PKT_DATA(p) + ETHERNET_HEADER_LEN,
-               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN, &pq);
+               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN);
     if (p->tcph != NULL) {
         printf("tcp header should be NULL for ip fragment, but it isn't\n");
         result = 0;
         goto end;
     }
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
 
     PacketCopyData(p, pkt2, sizeof(pkt2));
     DecodeIPV4(&tv, &dtv, p, GET_PKT_DATA(p) + ETHERNET_HEADER_LEN,
-               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN, &pq);
+               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN);
     if (p->tcph != NULL) {
         printf("tcp header should be NULL for ip fragment, but it isn't\n");
         result = 0;
         goto end;
     }
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
 
     PacketCopyData(p, pkt3, sizeof(pkt3));
     DecodeIPV4(&tv, &dtv, p, GET_PKT_DATA(p) + ETHERNET_HEADER_LEN,
-               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN, &pq);
+               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN);
     if (p->tcph != NULL) {
         printf("tcp header should be NULL for ip fragment, but it isn't\n");
         result = 0;
         goto end;
     }
-    Packet *tp = PacketDequeue(&pq);
+    Packet *tp = PacketDequeueNoLock(&tv.decode_pq);
     if (tp == NULL) {
         printf("Failed to get defragged pseudo packet\n");
         result = 0;
@@ -1331,12 +1356,12 @@ static int DecodeIPV4DefragTest01(void)
             goto end;
     }
 
-    PACKET_RECYCLE(tp);
+    PacketRecycle(tp);
     SCFree(tp);
 
 end:
     DefragDestroy();
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
     FlowShutdown();
     SCFree(p);
     return result;
@@ -1399,43 +1424,41 @@ static int DecodeIPV4DefragTest02(void)
         return 0;
     ThreadVars tv;
     DecodeThreadVars dtv;
-    PacketQueue pq;
     int result = 0;
 
     memset(&tv, 0, sizeof(ThreadVars));
     memset(&dtv, 0, sizeof(DecodeThreadVars));
-    memset(&pq, 0, sizeof(PacketQueue));
 
     FlowInitConfig(FLOW_QUIET);
     DefragInit();
 
     PacketCopyData(p, pkt1, sizeof(pkt1));
     DecodeIPV4(&tv, &dtv, p, GET_PKT_DATA(p) + ETHERNET_HEADER_LEN,
-               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN, &pq);
+               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN);
     if (p->tcph != NULL) {
         printf("tcp header should be NULL for ip fragment, but it isn't\n");
         goto end;
     }
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
 
     PacketCopyData(p, pkt2, sizeof(pkt2));
     DecodeIPV4(&tv, &dtv, p, GET_PKT_DATA(p) + ETHERNET_HEADER_LEN,
-               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN, &pq);
+               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN);
     if (p->tcph != NULL) {
         printf("tcp header should be NULL for ip fragment, but it isn't\n");
         goto end;
     }
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
 
     p->recursion_level = 3;
     PacketCopyData(p, pkt3, sizeof(pkt3));
     DecodeIPV4(&tv, &dtv, p, GET_PKT_DATA(p) + ETHERNET_HEADER_LEN,
-               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN, &pq);
+               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN);
     if (p->tcph != NULL) {
         printf("tcp header should be NULL for ip fragment, but it isn't\n");
         goto end;
     }
-    Packet *tp = PacketDequeue(&pq);
+    Packet *tp = PacketDequeueNoLock(&tv.decode_pq);
     if (tp == NULL) {
         printf("Failed to get defragged pseudo packet\n");
         goto end;
@@ -1463,12 +1486,12 @@ static int DecodeIPV4DefragTest02(void)
     }
 
     result = 1;
-    PACKET_RECYCLE(tp);
+    PacketRecycle(tp);
     SCFree(tp);
 
 end:
     DefragDestroy();
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
     FlowShutdown();
     SCFree(p);
     return result;
@@ -1526,19 +1549,17 @@ static int DecodeIPV4DefragTest03(void)
         return 0;
     ThreadVars tv;
     DecodeThreadVars dtv;
-    PacketQueue pq;
     int result = 1;
 
     memset(&tv, 0, sizeof(ThreadVars));
     memset(&dtv, 0, sizeof(DecodeThreadVars));
-    memset(&pq, 0, sizeof(PacketQueue));
 
     FlowInitConfig(FLOW_QUIET);
     DefragInit();
 
     PacketCopyData(p, pkt, sizeof(pkt));
     DecodeIPV4(&tv, &dtv, p, GET_PKT_DATA(p) + ETHERNET_HEADER_LEN,
-               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN, &pq);
+               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN);
     if (p->tcph == NULL) {
         printf("tcp header shouldn't be NULL, but it is\n");
         result = 0;
@@ -1549,38 +1570,38 @@ static int DecodeIPV4DefragTest03(void)
         result = 0;
         goto end;
     }
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
 
     PacketCopyData(p, pkt1, sizeof(pkt1));
     DecodeIPV4(&tv, &dtv, p, GET_PKT_DATA(p) + ETHERNET_HEADER_LEN,
-               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN, &pq);
+               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN);
     if (p->tcph != NULL) {
         printf("tcp header should be NULL for ip fragment, but it isn't\n");
         result = 0;
         goto end;
     }
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
 
     PacketCopyData(p, pkt2, sizeof(pkt2));
     DecodeIPV4(&tv, &dtv, p, GET_PKT_DATA(p) + ETHERNET_HEADER_LEN,
-               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN, &pq);
+               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN);
     if (p->tcph != NULL) {
         printf("tcp header should be NULL for ip fragment, but it isn't\n");
         result = 0;
         goto end;
     }
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
 
     PacketCopyData(p, pkt3, sizeof(pkt3));
     DecodeIPV4(&tv, &dtv, p, GET_PKT_DATA(p) + ETHERNET_HEADER_LEN,
-               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN, &pq);
+               GET_PKT_LEN(p) - ETHERNET_HEADER_LEN);
     if (p->tcph != NULL) {
         printf("tcp header should be NULL for ip fragment, but it isn't\n");
         result = 0;
         goto end;
     }
 
-    Packet *tp = PacketDequeue(&pq);
+    Packet *tp = PacketDequeueNoLock(&tv.decode_pq);
     if (tp == NULL) {
         printf("Failed to get defragged pseudo packet\n");
         result = 0;
@@ -1620,17 +1641,46 @@ static int DecodeIPV4DefragTest03(void)
             goto end;
     }
 
-    PACKET_RECYCLE(tp);
+    PacketRecycle(tp);
     SCFree(tp);
 
 end:
     DefragDestroy();
-    PACKET_RECYCLE(p);
+    PacketRecycle(p);
     FlowShutdown();
     SCFree(p);
     return result;
 }
 
+/**
+ */
+static int DecodeEthernetTestIPv4Opt(void)
+{
+    uint8_t raw_eth[] = {
+        0xae, 0x71, 0x00, 0x00, 0x00, 0x4b, 0x06, 0x90, 0x61, 0x02, 0x00, 0xcd, 0x88, 0x64, 0x11, 0x00,
+        0x15, 0x00, 0x80, 0x64, 0x00, 0x21, 0x4c, 0x00, 0x00, 0x30, 0x42, 0xd6, 0xff, 0xff, 0xbd, 0x2f,
+        0x02, 0x02, 0x00, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02,
+        0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02, 0x02,
+        0x01, 0x44, 0x05, 0x22, 0x02, 0x01
+    };
+
+    DefragInit();
+
+    Packet *p = PacketGetFromAlloc();
+    FAIL_IF_NULL(p);
+    ThreadVars tv;
+    DecodeThreadVars dtv;
+
+    memset(&dtv, 0, sizeof(DecodeThreadVars));
+    memset(&tv,  0, sizeof(ThreadVars));
+
+    DecodeEthernet(&tv, &dtv, p, raw_eth, sizeof(raw_eth));
+
+    SCFree(p);
+    DefragDestroy();
+    PASS;
+}
+
 #endif /* UNITTESTS */
 
 void DecodeIPV4RegisterTests(void)
@@ -1651,6 +1701,8 @@ void DecodeIPV4RegisterTests(void)
     UtRegisterTest("DecodeIPV4OptionsTSTest04", DecodeIPV4OptionsTSTest04);
     UtRegisterTest("DecodeIPV4OptionsSECTest01", DecodeIPV4OptionsSECTest01);
     UtRegisterTest("DecodeIPV4OptionsSECTest02", DecodeIPV4OptionsSECTest02);
+    UtRegisterTest("DecodeIPV4OptionsESECTest01", DecodeIPV4OptionsESECTest01);
+    UtRegisterTest("DecodeIPV4OptionsESECTest02", DecodeIPV4OptionsESECTest02);
     UtRegisterTest("DecodeIPV4OptionsLSRRTest01", DecodeIPV4OptionsLSRRTest01);
     UtRegisterTest("DecodeIPV4OptionsLSRRTest02", DecodeIPV4OptionsLSRRTest02);
     UtRegisterTest("DecodeIPV4OptionsLSRRTest03", DecodeIPV4OptionsLSRRTest03);
@@ -1674,6 +1726,7 @@ void DecodeIPV4RegisterTests(void)
     UtRegisterTest("DecodeIPV4DefragTest01", DecodeIPV4DefragTest01);
     UtRegisterTest("DecodeIPV4DefragTest02", DecodeIPV4DefragTest02);
     UtRegisterTest("DecodeIPV4DefragTest03", DecodeIPV4DefragTest03);
+    UtRegisterTest("DecodeEthernetTestIPv4Opt", DecodeEthernetTestIPv4Opt);
 #endif /* UNITTESTS */
 }
 /**
