@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2020 Open Information Security Foundation
+/* Copyright (C) 2007-2021 Open Information Security Foundation
  *
  * You can copy, redistribute or modify this Program under the terms of
  * the GNU General Public License version 2 as published by the Free
@@ -25,7 +25,6 @@
  */
 
 #include "suricata-common.h"
-#include "debug.h"
 #include "detect.h"
 #include "pkt-var.h"
 #include "conf.h"
@@ -70,21 +69,20 @@
 #include "stream-tcp-reassemble.h"
 
 typedef struct OutputFileCtx_ {
-    LogFileCtx *file_ctx;
     uint32_t file_cnt;
     HttpXFFCfg *xff_cfg;
     HttpXFFCfg *parent_xff_cfg;
-    OutputJsonCommonSettings cfg;
+    OutputJsonCtx *eve_ctx;
 } OutputFileCtx;
 
 typedef struct JsonFileLogThread_ {
     OutputFileCtx *filelog_ctx;
-    LogFileCtx *file_ctx;
-    MemBuffer *buffer;
+    OutputJsonThreadCtx *ctx;
 } JsonFileLogThread;
 
-JsonBuilder *JsonBuildFileInfoRecord(const Packet *p, const File *ff, const bool stored,
-        uint8_t dir, HttpXFFCfg *xff_cfg, OutputJsonCommonSettings *cfg)
+JsonBuilder *JsonBuildFileInfoRecord(const Packet *p, const File *ff, void *tx,
+        const uint64_t tx_id, const bool stored, uint8_t dir, HttpXFFCfg *xff_cfg,
+        OutputJsonCtx *eve_ctx)
 {
     enum OutputJsonLogDirection fdir = LOG_DIR_FLOW;
 
@@ -107,8 +105,8 @@ JsonBuilder *JsonBuildFileInfoRecord(const Packet *p, const File *ff, const bool
     int have_xff_ip = 0;
     char xff_buffer[XFF_MAXLEN];
     if ((xff_cfg != NULL) && !(xff_cfg->flags & XFF_DISABLED)) {
-        if (FlowGetAppProtocol(p->flow) == ALPROTO_HTTP) {
-            have_xff_ip = HttpXFFGetIPFromTx(p->flow, ff->txid, xff_cfg, xff_buffer, XFF_MAXLEN);
+        if (FlowGetAppProtocol(p->flow) == ALPROTO_HTTP1) {
+            have_xff_ip = HttpXFFGetIPFromTx(p->flow, tx_id, xff_cfg, xff_buffer, XFF_MAXLEN);
         }
         if (have_xff_ip && xff_cfg->flags & XFF_OVERWRITE) {
             if (p->flowflags & FLOW_PKT_TOCLIENT) {
@@ -120,31 +118,28 @@ JsonBuilder *JsonBuildFileInfoRecord(const Packet *p, const File *ff, const bool
         }
     }
 
-    JsonBuilder *js = CreateEveHeader(p, fdir, "fileinfo", &addr);
+    JsonBuilder *js = CreateEveHeader(p, fdir, "fileinfo", &addr, eve_ctx);
     if (unlikely(js == NULL))
         return NULL;
-    if (cfg != NULL) {
-        EveAddCommonOptions(cfg, p, p->flow, js);
-    }
 
     JsonBuilderMark mark = { 0, 0, 0 };
     switch (p->flow->alproto) {
-        case ALPROTO_HTTP:
+        case ALPROTO_HTTP1:
             jb_open_object(js, "http");
-            EveHttpAddMetadata(p->flow, ff->txid, js);
+            EveHttpAddMetadata(p->flow, tx_id, js);
             jb_close(js);
             break;
         case ALPROTO_SMTP:
             jb_get_mark(js, &mark);
             jb_open_object(js, "smtp");
-            if (EveSMTPAddMetadata(p->flow, ff->txid, js)) {
+            if (EveSMTPAddMetadata(p->flow, tx_id, js)) {
                 jb_close(js);
             } else {
                 jb_restore_mark(js, &mark);
             }
             jb_get_mark(js, &mark);
             jb_open_object(js, "email");
-            if (EveEmailAddMetadata(p->flow, ff->txid, js)) {
+            if (EveEmailAddMetadata(p->flow, tx_id, js)) {
                 jb_close(js);
             } else {
                 jb_restore_mark(js, &mark);
@@ -154,7 +149,7 @@ JsonBuilder *JsonBuildFileInfoRecord(const Packet *p, const File *ff, const bool
             /* rpc */
             jb_get_mark(js, &mark);
             jb_open_object(js, "rpc");
-            if (EveNFSAddMetadataRPC(p->flow, ff->txid, js)) {
+            if (EveNFSAddMetadataRPC(p->flow, tx_id, js)) {
                 jb_close(js);
             } else {
                 jb_restore_mark(js, &mark);
@@ -162,7 +157,7 @@ JsonBuilder *JsonBuildFileInfoRecord(const Packet *p, const File *ff, const bool
             /* nfs */
             jb_get_mark(js, &mark);
             jb_open_object(js, "nfs");
-            if (EveNFSAddMetadata(p->flow, ff->txid, js)) {
+            if (EveNFSAddMetadata(p->flow, tx_id, js)) {
                 jb_close(js);
             } else {
                 jb_restore_mark(js, &mark);
@@ -171,7 +166,7 @@ JsonBuilder *JsonBuildFileInfoRecord(const Packet *p, const File *ff, const bool
         case ALPROTO_SMB:
             jb_get_mark(js, &mark);
             jb_open_object(js, "smb");
-            if (EveSMBAddMetadata(p->flow, ff->txid, js)) {
+            if (EveSMBAddMetadata(p->flow, tx_id, js)) {
                 jb_close(js);
             } else {
                 jb_restore_mark(js, &mark);
@@ -180,7 +175,7 @@ JsonBuilder *JsonBuildFileInfoRecord(const Packet *p, const File *ff, const bool
         case ALPROTO_HTTP2:
             jb_get_mark(js, &mark);
             jb_open_object(js, "http");
-            if (EveHTTP2AddMetadata(p->flow, ff->txid, js)) {
+            if (EveHTTP2AddMetadata(p->flow, tx_id, js)) {
                 jb_close(js);
             } else {
                 jb_restore_mark(js, &mark);
@@ -191,7 +186,13 @@ JsonBuilder *JsonBuildFileInfoRecord(const Packet *p, const File *ff, const bool
     jb_set_string(js, "app_proto", AppProtoToString(p->flow->alproto));
 
     jb_open_object(js, "fileinfo");
-    EveFileInfo(js, ff, stored);
+    if (stored) {
+        // the file has just been stored on disk cf OUTPUT_FILEDATA_FLAG_CLOSE
+        // but the flag is not set until the loggers have been called
+        EveFileInfo(js, ff, tx_id, ff->flags | FILE_STORED);
+    } else {
+        EveFileInfo(js, ff, tx_id, ff->flags);
+    }
     jb_close(js);
 
     /* xff header */
@@ -206,24 +207,22 @@ JsonBuilder *JsonBuildFileInfoRecord(const Packet *p, const File *ff, const bool
  *  \internal
  *  \brief Write meta data on a single line json record
  */
-static void FileWriteJsonRecord(JsonFileLogThread *aft, const Packet *p,
-                                const File *ff, uint8_t dir)
+static void FileWriteJsonRecord(JsonFileLogThread *aft, const Packet *p, const File *ff, void *tx,
+        const uint64_t tx_id, uint8_t dir, OutputJsonCtx *eve_ctx)
 {
-    HttpXFFCfg *xff_cfg = aft->filelog_ctx->xff_cfg != NULL ?
-        aft->filelog_ctx->xff_cfg : aft->filelog_ctx->parent_xff_cfg;;
-    JsonBuilder *js = JsonBuildFileInfoRecord(
-            p, ff, ff->flags & FILE_STORED ? true : false, dir, xff_cfg, &aft->filelog_ctx->cfg);
+    HttpXFFCfg *xff_cfg = aft->filelog_ctx->xff_cfg != NULL ? aft->filelog_ctx->xff_cfg
+                                                            : aft->filelog_ctx->parent_xff_cfg;
+    JsonBuilder *js = JsonBuildFileInfoRecord(p, ff, tx, tx_id, false, dir, xff_cfg, eve_ctx);
     if (unlikely(js == NULL)) {
         return;
     }
 
-    MemBufferReset(aft->buffer);
-    OutputJsonBuilderBuffer(js, aft->file_ctx, &aft->buffer);
+    OutputJsonBuilderBuffer(js, aft->ctx);
     jb_free(js);
 }
 
-static int JsonFileLogger(ThreadVars *tv, void *thread_data, const Packet *p,
-                          const File *ff, uint8_t dir)
+static int JsonFileLogger(ThreadVars *tv, void *thread_data, const Packet *p, const File *ff,
+        void *tx, const uint64_t tx_id, uint8_t dir)
 {
     SCEnter();
     JsonFileLogThread *aft = (JsonFileLogThread *)thread_data;
@@ -232,7 +231,7 @@ static int JsonFileLogger(ThreadVars *tv, void *thread_data, const Packet *p,
 
     SCLogDebug("ff %p", ff);
 
-    FileWriteJsonRecord(aft, p, ff, dir);
+    FileWriteJsonRecord(aft, p, ff, tx, tx_id, dir, aft->filelog_ctx->eve_ctx);
     return 0;
 }
 
@@ -249,15 +248,10 @@ static TmEcode JsonFileLogThreadInit(ThreadVars *t, const void *initdata, void *
         goto error_exit;
     }
 
-    aft->buffer = MemBufferCreateNew(JSON_OUTPUT_BUFFER_SIZE);
-    if (aft->buffer == NULL) {
-        goto error_exit;
-    }
-
-    /* Use the Ouptut Context (file pointer and mutex) */
+    /* Use the Output Context (file pointer and mutex) */
     aft->filelog_ctx = ((OutputCtx *)initdata)->data;
-    aft->file_ctx = LogFileEnsureExists(aft->filelog_ctx->file_ctx, t->id);
-    if (!aft->file_ctx) {
+    aft->ctx = CreateEveThreadCtx(t, aft->filelog_ctx->eve_ctx);
+    if (!aft->ctx) {
         goto error_exit;
     }
 
@@ -265,9 +259,6 @@ static TmEcode JsonFileLogThreadInit(ThreadVars *t, const void *initdata, void *
     return TM_ECODE_OK;
 
 error_exit:
-    if (aft->buffer != NULL) {
-        MemBufferFree(aft->buffer);
-    }
     SCFree(aft);
     return TM_ECODE_FAILED;
 }
@@ -279,7 +270,8 @@ static TmEcode JsonFileLogThreadDeinit(ThreadVars *t, void *data)
         return TM_ECODE_OK;
     }
 
-    MemBufferFree(aft->buffer);
+    FreeEveThreadCtx(aft->ctx);
+
     /* clear memory */
     memset(aft, 0, sizeof(JsonFileLogThread));
 
@@ -316,9 +308,6 @@ static OutputInitResult OutputFileLogInitSub(ConfNode *conf, OutputCtx *parent_c
         return result;
     }
 
-    output_file_ctx->file_ctx = ojc->file_ctx;
-    output_file_ctx->cfg = ojc->cfg;
-
     if (conf) {
         const char *force_filestore = ConfNodeLookupChildValue(conf, "force-filestore");
         if (force_filestore != NULL && ConfValIsTrue(force_filestore)) {
@@ -344,6 +333,7 @@ static OutputInitResult OutputFileLogInitSub(ConfNode *conf, OutputCtx *parent_c
         output_file_ctx->parent_xff_cfg = ojc->xff_cfg;
     }
 
+    output_file_ctx->eve_ctx = ojc;
     output_ctx->data = output_file_ctx;
     output_ctx->DeInit = OutputFileLogDeinitSub;
 
